@@ -1,6 +1,9 @@
 #!/usr/bin/env python
 
-import os, os.path as op, sys
+from __future__ import print_function
+import os
+import os.path as op
+import sys
 from datetime import date
 
 from opster import Dispatcher
@@ -11,7 +14,7 @@ from nomad.engine import DBError
 from nomad.utils import abort, NomadError, NomadIniNotFound
 
 
-__version__ = '1.9'
+__version__ = '2.2'
 
 
 GLOBAL = [
@@ -30,19 +33,21 @@ EXAMPLE_INI = '''
 def getconfig(func):
     if func.__name__.startswith('help') or func.__name__ in ('version',):
         return func
+
     def inner(*args, **kwargs):
         try:
             repo = Repository(kwargs['config'], kwargs['define'])
-        except NomadIniNotFound, e:
+        except NomadIniNotFound as e:
             sys.stderr.write("Create '%s' to use nomad, example:\n%s\n" %
                              (e, EXAMPLE_INI))
             abort('config file not found')
-        except (IOError, NomadError), e:
+        except (IOError, NomadError) as e:
             abort(e)
 
         return func(repo=repo, *args, **kwargs)
     return inner
 
+
 app = Dispatcher(globaloptions=GLOBAL, middleware=getconfig)
 
 
@@ -52,30 +57,32 @@ def init(**opts):
     '''
     try:
         opts['repo'].init_db()
-    except DBError, e:
+    except DBError as e:
         abort(e)
-    print 'Versioning table initialized successfully'
+    print('Versioning table initialized successfully')
 
 
 @app.command(name='list', aliases=('ls',))
 def list_(all=('a', False, 'show all migrations (default: only non-applied)'),
-         **opts):
+          **opts):
     '''List migrations
     '''
     repo = opts['repo']
-    for m in repo.available:
-        if m in repo.applied:
+    all_migrations = repo.available + [m for m in repo.applied
+                                       if m not in repo.available]
+    for m in all_migrations:
+        if m not in repo.available:
+            print(colored(m, 'red') + ' (not on disk)')
+        elif m in repo.applied:
             if all:
                 cprint(m, 'magenta')
         else:
-            out = colored(m, 'green')
-            deps = []
-            for dep in m.dependencies:
-                if dep not in repo.applied:
-                    deps.append(dep)
+            out = colored(str(m), 'green')
+            deps = ', '.join(str(dep) for dep in m.dependencies
+                             if dep not in repo.applied)
             if deps:
-                out += ' (%s)' % ', '.join(map(str, deps))
-            print out
+                out += ' (%s)' % deps
+            print(out)
 
 
 @app.command()
@@ -94,7 +101,7 @@ def create(name,
     path = op.join(repo.path, name)
     try:
         os.mkdir(path)
-    except OSError, e:
+    except OSError as e:
         if e.errno == 17:
             abort('directory %s already exists' % path)
         raise
@@ -110,14 +117,17 @@ def create(name,
 def apply(all=('a', False, 'apply all available migrations'),
           init=('', False, 'init if not initialized yet'),
           env=('e', [], 'list of additional environment variables'),
+          fake=('', False, 'record migration as applied, but do nothing'),
           *names,
           **opts):
     '''Apply migration and all of it dependencies
 
     You can pass additional environment variables to your migrations scripts
-    using option `-e` (in addition to NOMAD_DBURL, which is passed by default):
+    using option `-e` (in addition to `NOMAD_<confvar>`s, which are passed by
+    default):
 
         nomad apply -e ONE=one -e TWO=two -a
+
     '''
     repo = opts['repo']
     if init:
@@ -127,7 +137,7 @@ def apply(all=('a', False, 'apply all available migrations'),
             pass
 
     if names:
-        migrations = map(repo.get, names)
+        migrations = [repo.get(x) for x in names]
     elif all:
         migrations = [x for x in repo.available if x not in repo.applied]
     else:
@@ -137,14 +147,14 @@ def apply(all=('a', False, 'apply all available migrations'),
         env = dict(x.split('=', 1) for x in env)
 
     for m in migrations:
-        if m in repo.applied:
+        if m.applied:
             abort('migration %s is already applied' % m)
+
     for m in migrations:
-        if not m.applied:
-            try:
-                m.apply(env=env)
-            except DBError, e:
-                abort('cannot apply migration %s: %s' % (m, e))
+        try:
+            m.apply(env=env, fake=fake)
+        except DBError as e:
+            abort('cannot apply migration %s: %s' % (m, e))
 
 
 @app.command()
@@ -152,20 +162,20 @@ def info(**opts):
     '''Show information about repository
     '''
     repo = opts['repo']
-    print '%s:' % repo
-    print '  %s' % repo.engine
+    print('%s:' % repo)
+    print('  %s' % repo.engine)
     try:
-        print '  %s applied' % len(repo.applied)
-        print '  %s unapplied' % (len(repo.available) - len(repo.applied))
+        print('  %s applied' % len(repo.applied))
+        print('  %s unapplied' % (len(repo.available) - len(repo.applied)))
     except DBError:
-        print '  Uninitialized repository'
+        print('  Uninitialized repository')
 
 
 @app.command()
 def version(**opts):
     '''Show nomad version
     '''
-    print 'Nomad v%s' % __version__
+    print('Nomad v%s' % __version__)
 
 
 if __name__ == '__main__':
