@@ -1,27 +1,21 @@
-import urlparse
+import urllib, urllib.parse
 
 from nomad.engine import BaseEngine, DBError
 
 
-def path2dict(p, **renames):
-    '''Convert urlparse result to dict
-    '''
-    result = {}
-    for k in 'hostname port path username password'.split():
-        value = getattr(p, k, None)
-        if not value:
-            continue
-        if k == 'path':
-            value = value.lstrip('/')
-        result[renames.get(k, k)] = value
-    return result
+def unq(s):
+    if isinstance(s, str):
+        return urllib.parse.unquote(s)
+    return s
 
 
 class Connection(object):
     exc = None
 
-    @property
-    def connection(self):
+    def __init__(self):
+        self.connection = self.connect()
+
+    def connect(self):
         raise NotImplementedError()
 
     def prepare(self, statement):
@@ -35,13 +29,16 @@ class Connection(object):
         c = self.connection.cursor()
         try:
             c.execute(statement, args)
-        except self.exc, e:
+        except self.exc as e:
             raise DBError(e)
 
         data = self.fetch(c)
         c.close()
         return data
 
+    def nobegin(self):
+        pass
+
     def begin(self):
         pass
 
@@ -61,28 +58,31 @@ class Sqlite(Connection):
         import sqlite3
         self.module = sqlite3
         self.exc = sqlite3.Error
+        Connection.__init__(self)
 
-    @property
-    def connection(self):
-        if not self._conn:
-            self._conn = self.module.connect(self.path)
-        return self._conn
+    def connect(self):
+        return self.module.connect(self.path)
 
 
 class Mysql(Connection):
     _conn = None
-    def __init__(self, path):
-        self.parameters = path2dict(path, hostname='host',
-                                    password='passwd', path='db', username='user')
+
+    def __init__(self, url):
+        self.parameters = {'host':   unq(url.hostname),
+                           'port':   url.port,
+                           'db':     unq(url.path.lstrip('/')),
+                           'user':   unq(url.username),
+                           'passwd': unq(url.password)}
+        for k, v in self.parameters.items():
+            if not v:
+                del self.parameters[k]
         import MySQLdb
         self.module = MySQLdb
         self.exc = MySQLdb.MySQLError
+        Connection.__init__(self)
 
-    @property
-    def connection(self):
-        if not self._conn:
-            self._conn = self.module.connect(**self.parameters)
-        return self._conn
+    def connect(self):
+        return self.module.connect(**self.parameters)
 
     def prepare(self, statement):
         return statement.replace('?', '%s')
@@ -99,17 +99,29 @@ class Mysql(Connection):
 
 class Pgsql(Connection):
     _conn = None
-    def __init__(self, path):
-        self.parameters = path2dict(path, hostname='host', path='database', username='user')
+
+    def __init__(self, url):
+        self.parameters = {'host':     unq(url.hostname),
+                           'port':     url.port,
+                           'database': unq(url.path.lstrip('/')),
+                           'user':     unq(url.username),
+                           'password': unq(url.password)}
+        if url.query:
+            self.options = url.query.split('&')
+        else:
+            self.options = ['statement_timeout=1000', 'lock_timeout=500']
+
         import psycopg2
+        import psycopg2.extensions
         self.module = psycopg2
         self.exc = psycopg2.Error
+        Connection.__init__(self)
 
-    @property
-    def connection(self):
-        if not self._conn:
-            self._conn = self.module.connect(**self.parameters)
-        return self._conn
+    def connect(self):
+        try:
+            return self.module.connect(**self.parameters)
+        except self.module.OperationalError as e:
+            raise DBError('psycopg2: %s' % e)
 
     def prepare(self, statement):
         return statement.replace('?', '%s')
@@ -122,6 +134,17 @@ class Pgsql(Connection):
         except self.module.ProgrammingError:
             return []
 
+    def begin(self):
+        self.connection.set_isolation_level(
+            self.module.extensions.ISOLATION_LEVEL_DEFAULT)
+        c = self.connection.cursor()
+        c.executemany(['BEGIN'] + ['set ' + x for x in self.options], [])
+        c.close()
+
+    def nobegin(self):
+        self.connection.set_isolation_level(
+            self.module.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
+
 
 CONNECTORS = {'sqlite': Sqlite, 'mysql': Mysql,
               'pgsql': Pgsql, 'postgresql': Pgsql, 'postgres': Pgsql}
@@ -129,12 +152,12 @@ CONNECTORS = {'sqlite': Sqlite, 'mysql': Mysql,
 
 class DBEngine(BaseEngine):
     def connect(self):
-        p = urlparse.urlparse(self.url)
+        p = urllib.parse.urlparse(self.url)
         if p.scheme not in CONNECTORS:
             raise DBError('scheme "%s" not supported' % p.scheme)
         return CONNECTORS[p.scheme](p)
 
-    def query(self, statement, *args):
+    def query(self, statement, *args, **kwargs):
         return self.connection.query(statement, *args)
 
 
