@@ -1,26 +1,38 @@
+from __future__ import print_function
 import sys
 import re
-import os, os.path as op
+import os
+import os.path as op
 import shlex
-import imp
 import json
+from functools import reduce
 from subprocess import Popen, PIPE
 from configparser import ConfigParser, ExtendedInterpolation
 
 from termcolor import cprint
 
+
+try:
+    import importlib
+except ImportError:
+    importlib = None
+    import imp
+
+
 if sys.version_info[0] == 3:
     shell_split = shlex.split
 else:
     def shell_split(cmd):
         return shlex.split(cmd.encode('utf-8'))
 
+
 NUM_RE = re.compile('(\d+)')
 
 
 class NomadError(Exception):
     pass
 
+
 class NomadIniNotFound(Exception):
     pass
 
@@ -53,26 +65,34 @@ def shsplit(s):
 
 
 def humankey(fn):
-  '''Turn a string into a list of substrings and numbers.
+    '''Turn a string into a list of substrings and numbers.
 
-  This can be used as a key function for ``sorted``::
+    This can be used as a key function for ``sorted``::
+
+        >>> s = lambda *x: list(sorted(x, key=humankey))
+        >>> print(s('up-1', 'up-5', 'up-15', 'up-50'))
+        ['up-1', 'up-5', 'up-15', 'up-50']
+        >>> print(s('up-1.sql', 'up.sql', 'up1.sql'))
+        ['up.sql', 'up1.sql', 'up-1.sql']
+        >>> print(s('up.rb', 'up.py')) # check extension sorting
+        ['up.py', 'up.rb']
+    '''
+    fn, ext = os.path.splitext(fn)
+    return [int(s) if s.isdigit() else s for s in NUM_RE.split(fn)], ext
 
-    >>> s = lambda *x: list(sorted(x, key=humankey))
-    >>> print s('up-1', 'up-5', 'up-15', 'up-50')
-    ['up-1', 'up-5', 'up-15', 'up-50']
-    >>> print s('up-1.sql', 'up.sql', 'up1.sql')
-    ['up.sql', 'up1.sql', 'up-1.sql']
-    >>> print s('up.rb', 'up.py') # check extension sorting
-    ['up.py', 'up.rb']
-  '''
-  fn, ext = os.path.splitext(fn)
-  return [int(s) if s.isdigit() else s for s in NUM_RE.split(fn)], ext
 
 
 def loadpath(path):
     modname = 'nomad_url_%s' % path.replace('/', '_').replace('.', '_')
     path = op.expanduser(op.expandvars(path))
-    if op.isdir(path):
+    if importlib:
+        if not path.endswith('.py'):
+            path = op.join(path, '__init__.py')
+        spec = importlib.util.spec_from_file_location(modname, path)
+        mod = importlib.util.module_from_spec(spec)
+        spec.loader.exec_module(mod)
+        return mod
+    elif op.isdir(path):
         # path/__init__.py
         d, f = op.split(path.rstrip('/'))
         fd, fpath, desc = imp.find_module(f, [d])
@@ -85,8 +105,8 @@ def clean_sql(sql):
     return '\n'.join(x for x in sql.split('\n')
                      if not x.strip().startswith('--'))
 
-### URL retrievers
 
+# URL retrievers
 def get_python(path):
     pypath, attr = path.split(':')
     if '/' in pypath or pypath.endswith('.py'):
@@ -134,7 +154,7 @@ def get_yaml(path):
     except ImportError:
         abort('Please, install PyYAML to parse YAML config.')
     fn, path = path.split(':')
-    obj = yaml.load(open(fn))
+    obj = yaml.load(open(fn), Loader=yaml.BaseLoader)
     path = map(lambda x: int(x) if x.isdigit() else x, path.split('.'))
     return reduce(lambda x, y: x[y], path, obj)
 
